@@ -1,6 +1,6 @@
 _POWERLINE_SOURCED="$_"
 _powerline_columns_fallback() {
-	if which stty >/dev/null ; then
+	if command -v stty >/dev/null ; then
 		# Ksh does not have “local” built-in
 		_powerline_cols="$(stty size 2>/dev/null)"
 		if ! test -z "$_powerline_cols" ; then
@@ -104,13 +104,18 @@ _powerline_set_command() {
 	fi
 }
 
+_powerline_tmux_pane() {
+	echo "${TMUX_PANE:-`TMUX="$_POWERLINE_TMUX" tmux display -p "#D"`}" | \
+		tr -d ' %'
+}
+
 _powerline_tmux_setenv() {
-	TMUX="$_POWERLINE_TMUX" tmux setenv -g TMUX_"$1"_`tmux display -p "#D" | tr -d %` "$2"
+	TMUX="$_POWERLINE_TMUX" tmux setenv -g TMUX_"$1"_`_powerline_tmux_pane` "$2"
 	TMUX="$_POWERLINE_TMUX" tmux refresh -S
 }
 
 _powerline_tmux_set_pwd() {
-	if test "x$_POWERLINE_SAVED_PWD" != "x$PWD" ; then
+	if test "$_POWERLINE_SAVED_PWD" != "$PWD" ; then
 		_POWERLINE_SAVED_PWD="$PWD"
 		_powerline_tmux_setenv PWD "$PWD"
 	fi
@@ -215,7 +220,7 @@ _powerline_init_tmux_support() {
 }
 
 if test -z "${POWERLINE_CONFIG_COMMAND}" ; then
-	if which powerline-config >/dev/null ; then
+	if command -v powerline-config >/dev/null ; then
 		POWERLINE_CONFIG_COMMAND=powerline-config
 	else
 		POWERLINE_CONFIG_COMMAND="$(dirname "$_POWERLINE_SOURCED")/../../../scripts/powerline-config"
