@@ -1,10 +1,7 @@
 # vim:fileencoding=utf-8:noet
 from __future__ import (unicode_literals, division, absolute_import, print_function)
 
-import subprocess
-from traceback import print_exc
-
-__version__ = "2.8.2"
+__version__ = "2.8.3"
 
 def get_version():
 	return __version__
