@@ -1,6 +1,9 @@
+from powerline.bindings.ipython.since_7 import PowerlinePrompts
 import os
 c = get_config()
-c.InteractiveShellApp.extensions = ['powerline.bindings.ipython.post_0_11']
+c.TerminalInteractiveShell.simple_prompt = False
+c.TerminalIPythonApp.display_banner = False
+c.TerminalInteractiveShell.prompts_class = PowerlinePrompts
 c.TerminalInteractiveShell.autocall = 1
 c.Powerline.config_paths = [os.path.abspath('powerline/config_files')]
 c.Powerline.theme_overrides = {
